=== modified file 'drizzled/function/func.cc'
--- drizzled/function/func.cc	2010-12-25 04:38:28 +0000
+++ drizzled/function/func.cc	2011-01-06 05:17:09 +0000
@@ -62,7 +62,8 @@
 
 Item_func::Item_func(List<Item> &list) :
   _session(*current_session),
-  allowed_arg_cols(1)
+  allowed_arg_cols(1),
+  const_item_cache(false)
 {
   collation.set(DERIVATION_SYSCONST);
   set_arguments(list);
@@ -134,7 +135,7 @@
   unsigned char buff[STACK_BUFF_ALLOC];      // Max argument in function
   session->session_marker= 0;
   used_tables_cache= not_null_tables_cache= 0;
-  const_item_cache=1;
+  const_item_cache= true;
 
   if (check_stack_overrun(session, STACK_MIN_SIZE, buff))
     return true;        // Fatal error if flag is set!
@@ -189,7 +190,7 @@
   Item **arg,**arg_end;
 
   used_tables_cache= not_null_tables_cache= 0;
-  const_item_cache=1;
+  const_item_cache= false;
 
   if (arg_count)
   {
@@ -354,7 +355,7 @@
 void Item_func::update_used_tables()
 {
   used_tables_cache=0;
-  const_item_cache=1;
+  const_item_cache= true;
   for (uint32_t i=0 ; i < arg_count ; i++)
   {
     args[i]->update_used_tables();

=== modified file 'drizzled/function/func.h'
--- drizzled/function/func.h	2010-12-25 04:38:28 +0000
+++ drizzled/function/func.h	2011-01-06 05:17:09 +0000
@@ -67,7 +67,8 @@
 
   Item_func(void):
     _session(*current_session),
-    allowed_arg_cols(1), arg_count(0)
+    allowed_arg_cols(1), arg_count(0),
+    const_item_cache(false)
   {
     with_sum_func= 0;
     collation.set(DERIVATION_SYSCONST);
@@ -75,7 +76,8 @@
 
   Item_func(Item *a):
     _session(*current_session),
-    allowed_arg_cols(1), arg_count(1)
+    allowed_arg_cols(1), arg_count(1),
+    const_item_cache(false)
   {
     args= tmp_arg;
     args[0]= a;
@@ -85,7 +87,8 @@
   
   Item_func(Item *a,Item *b):
     _session(*current_session),
-    allowed_arg_cols(1), arg_count(2)
+    allowed_arg_cols(1), arg_count(2),
+    const_item_cache(false)
   {
     args= tmp_arg;
     args[0]= a; args[1]= b;
@@ -95,7 +98,8 @@
   
   Item_func(Item *a,Item *b,Item *c):
     _session(*current_session),
-    allowed_arg_cols(1)
+    allowed_arg_cols(1),
+    const_item_cache(false)
   {
     arg_count= 0;
     if ((args= (Item**) memory::sql_alloc(sizeof(Item*)*3)))
@@ -109,7 +113,8 @@
   
   Item_func(Item *a,Item *b,Item *c,Item *d):
     _session(*current_session),
-    allowed_arg_cols(1)
+    allowed_arg_cols(1),
+    const_item_cache(false)
   {
     arg_count= 0;
     if ((args= (Item**) memory::sql_alloc(sizeof(Item*)*4)))
@@ -124,7 +129,8 @@
   
   Item_func(Item *a,Item *b,Item *c,Item *d,Item* e):
     _session(*current_session),
-    allowed_arg_cols(1)
+    allowed_arg_cols(1),
+    const_item_cache(false)
   {
     arg_count= 5;
     if ((args= (Item**) memory::sql_alloc(sizeof(Item*)*5)))
@@ -188,6 +194,11 @@
 
   bool is_null();
 
+  virtual bool deterministic() const
+  {
+    return false;
+  }
+
   void signal_divide_by_null();
 
   virtual Field *tmp_table_field() { return result_field; }

=== modified file 'drizzled/function_container.cc'
--- drizzled/function_container.cc	2010-12-18 04:43:40 +0000
+++ drizzled/function_container.cc	2011-01-06 05:17:09 +0000
@@ -26,14 +26,14 @@
 
 using namespace std;
 
-static drizzled::NativeFunctionsMap native_functions_map;
+static drizzled::FunctionContainer::Map _functions_map;
 
 namespace drizzled
 {
 
-NativeFunctionsMap &FunctionContainer::getMap()
+FunctionContainer::Map &FunctionContainer::getMap()
 {
-  return native_functions_map;
+  return _functions_map;
 }
 
 } /* namespace drizzled */

=== modified file 'drizzled/function_container.h'
--- drizzled/function_container.h	2010-12-18 04:43:40 +0000
+++ drizzled/function_container.h	2011-01-06 05:17:09 +0000
@@ -27,12 +27,13 @@
 
 namespace drizzled {
 
-struct Native_func_registry;
-typedef boost::unordered_map<std::string, Native_func_registry *, util::insensitive_hash, util::insensitive_equal_to> NativeFunctionsMap;
+class Create_func;
 
 class FunctionContainer {
 public:
-  static NativeFunctionsMap &getMap();
+  typedef boost::unordered_map<std::string, Create_func *, util::insensitive_hash, util::insensitive_equal_to> Map;
+
+  static Map &getMap();
 };
 
 } /* namepsace drizzled */

=== modified file 'drizzled/generator/functions.cc'
--- drizzled/generator/functions.cc	2010-09-10 22:40:11 +0000
+++ drizzled/generator/functions.cc	2011-01-06 05:17:09 +0000
@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include "drizzled/generator.h"
+#include "drizzled/function_container.h"
 #include "drizzled/plugin/function.h"
 #include "drizzled/session.h"
 
@@ -34,7 +35,19 @@
 Functions::Functions(Session &arg) :
   session(arg)
 {
-  udf_iter= plugin::Function::getMap().begin();
+  function_list.reserve(plugin::Function::getMap().size() + FunctionContainer::getMap().size());
+
+  std::transform(FunctionContainer::getMap().begin(),
+                 FunctionContainer::getMap().end(),
+                 std::back_inserter(function_list),
+                 boost::bind(&FunctionContainer::Map::value_type::first, _1) );
+
+  std::transform(plugin::Function::getMap().begin(),
+                 plugin::Function::getMap().end(),
+                 std::back_inserter(function_list),
+                 boost::bind(&plugin::Function::Map::value_type::first, _1) );
+
+  iter= function_list.begin();
 }
 
 } /* namespace generator */

=== modified file 'drizzled/generator/functions.h'
--- drizzled/generator/functions.h	2010-09-10 22:40:11 +0000
+++ drizzled/generator/functions.h	2011-01-06 05:17:09 +0000
@@ -29,8 +29,10 @@
 class Functions
 {
   Session &session;
+  typedef std::vector <std::string> vector;
   std::string function_name;
-  plugin::Function::UdfMap::const_iterator udf_iter;
+  vector function_list;
+  vector::iterator iter;
 
 public:
 
@@ -38,11 +40,11 @@
 
   operator std::string*()
   {
-    if (udf_iter == plugin::Function::getMap().end())
+    if (iter == function_list.end())
       return NULL;
 
-    function_name= (*udf_iter).first;
-    udf_iter++;
+    function_name= *iter;
+    iter++;
 
     return &function_name;
   }

=== modified file 'drizzled/item.cc'
--- drizzled/item.cc	2010-12-28 04:57:17 +0000
+++ drizzled/item.cc	2011-01-06 05:17:09 +0000
@@ -823,7 +823,7 @@
   {
     Item_subselect *prev_subselect_item= previous_select->master_unit()->item;
     prev_subselect_item->used_tables_cache|= OUTER_REF_TABLE_BIT;
-    prev_subselect_item->const_item_cache= 0;
+    prev_subselect_item->const_item_cache= false;
   }
   {
     Item_subselect *prev_subselect_item= previous_select->master_unit()->item;
@@ -838,7 +838,7 @@
     }
     else
       prev_subselect_item->used_tables_cache|= found_field->getTable()->map;
-    prev_subselect_item->const_item_cache= 0;
+    prev_subselect_item->const_item_cache= false;
     mark_as_dependent(session, last_select, current_sel, resolved_item,
                       dependent);
   }

=== modified file 'drizzled/item/cmpfunc.cc'
--- drizzled/item/cmpfunc.cc	2010-12-27 21:05:56 +0000
+++ drizzled/item/cmpfunc.cc	2011-01-06 05:17:09 +0000
@@ -3880,7 +3880,7 @@
   void *orig_session_marker= session->session_marker;
   unsigned char buff[sizeof(char*)];			// Max local vars in function
   not_null_tables_cache= used_tables_cache= 0;
-  const_item_cache= 1;
+  const_item_cache= true;
 
   if (functype() == COND_OR_FUNC)
     session->session_marker= 0;
@@ -3955,7 +3955,7 @@
   Item *item;
 
   used_tables_cache=0;
-  const_item_cache=1;
+  const_item_cache= true;
 
   and_tables_cache= ~(table_map) 0; // Here and below we do as fix_fields does
   not_null_tables_cache= 0;
@@ -4144,7 +4144,7 @@
   Item *item;
 
   used_tables_cache=0;
-  const_item_cache=1;
+  const_item_cache= true;
   while ((item=li++))
   {
     item->update_used_tables();
@@ -4888,7 +4888,7 @@
 Item_equal::Item_equal(Item_field *f1, Item_field *f2)
   : item::function::Boolean(), const_item(0), eval_item(0), cond_false(0)
 {
-  const_item_cache= 0;
+  const_item_cache= false;
   fields.push_back(f1);
   fields.push_back(f2);
 }
@@ -4896,7 +4896,7 @@
 Item_equal::Item_equal(Item *c, Item_field *f)
   : item::function::Boolean(), eval_item(0), cond_false(0)
 {
-  const_item_cache= 0;
+  const_item_cache= false;
   fields.push_back(f);
   const_item= c;
 }
@@ -4905,7 +4905,7 @@
 Item_equal::Item_equal(Item_equal *item_equal)
   : item::function::Boolean(), eval_item(0), cond_false(0)
 {
-  const_item_cache= 0;
+  const_item_cache= false;
   List_iterator_fast<Item_field> li(item_equal->fields);
   Item_field *item;
   while ((item= li++))
@@ -4929,7 +4929,7 @@
   func->set_cmp_func();
   func->quick_fix_field();
   if ((cond_false= !func->val_int()))
-    const_item_cache= 1;
+    const_item_cache= true;
 }
 
 void Item_equal::add(Item_field *f)
@@ -5075,7 +5075,7 @@
   List_iterator_fast<Item_field> li(fields);
   Item *item;
   not_null_tables_cache= used_tables_cache= 0;
-  const_item_cache= 0;
+  const_item_cache= false;
   while ((item= li++))
   {
     table_map tmp_table_map;

=== modified file 'drizzled/item/create.cc'
--- drizzled/item/create.cc	2010-12-27 19:18:58 +0000
+++ drizzled/item/create.cc	2011-01-06 05:17:09 +0000
@@ -2047,7 +2047,7 @@
   {
     func_name.assign(func->name.str, func->name.length);
 
-    FunctionContainer::getMap()[func_name]= func;
+    FunctionContainer::getMap()[func_name]= func->builder;
   }
 
   return 0;
@@ -2057,18 +2057,16 @@
 Create_func *
 find_native_function_builder(LEX_STRING name)
 {
-  Native_func_registry *func;
   Create_func *builder= NULL;
 
   string func_name(name.str, name.length);
 
-  NativeFunctionsMap::iterator func_iter=
+  FunctionContainer::Map::iterator func_iter=
     FunctionContainer::getMap().find(func_name);
 
   if (func_iter != FunctionContainer::getMap().end())
   {
-    func= (*func_iter).second;
-    builder= func->builder;
+    builder= (*func_iter).second;
   }
 
   return builder;

=== modified file 'drizzled/item/field.cc'
--- drizzled/item/field.cc	2010-12-26 22:31:33 +0000
+++ drizzled/item/field.cc	2011-01-06 05:17:09 +0000
@@ -538,7 +538,7 @@
         if (*from_field != view_ref_found)
         {
           prev_subselect_item->used_tables_cache|= (*from_field)->getTable()->map;
-          prev_subselect_item->const_item_cache= 0;
+          prev_subselect_item->const_item_cache= false;
           set_field(*from_field);
           if (!last_checked_context->select_lex->having_fix_field &&
               select->group_list.elements &&
@@ -626,7 +626,7 @@
       case it does not matter which used tables bits we set)
     */
     prev_subselect_item->used_tables_cache|= OUTER_REF_TABLE_BIT;
-    prev_subselect_item->const_item_cache= 0;
+    prev_subselect_item->const_item_cache= false;
   }
 
   assert(ref != 0);

=== modified file 'drizzled/item/ref.cc'
--- drizzled/item/ref.cc	2010-12-26 22:31:33 +0000
+++ drizzled/item/ref.cc	2011-01-06 05:17:09 +0000
@@ -247,7 +247,7 @@
                        cached_table->select_lex != outer_context->select_lex);
             }
             prev_subselect_item->used_tables_cache|= from_field->getTable()->map;
-            prev_subselect_item->const_item_cache= 0;
+            prev_subselect_item->const_item_cache= false;
             break;
           }
         }
@@ -255,7 +255,7 @@
 
         /* Reference is not found => depend on outer (or just error). */
         prev_subselect_item->used_tables_cache|= OUTER_REF_TABLE_BIT;
-        prev_subselect_item->const_item_cache= 0;
+        prev_subselect_item->const_item_cache= false;
 
         outer_context= outer_context->outer_context;
       } while (outer_context);

=== modified file 'drizzled/item/row.cc'
--- drizzled/item/row.cc	2010-10-02 21:15:42 +0000
+++ drizzled/item/row.cc	2011-01-06 05:17:09 +0000
@@ -99,7 +99,7 @@
   Item::cleanup();
   /* Reset to the original values */
   used_tables_cache= 0;
-  const_item_cache= 1;
+  const_item_cache= true;
   with_null= 0;
 
   return;
@@ -118,7 +118,7 @@
 void Item_row::update_used_tables()
 {
   used_tables_cache= 0;
-  const_item_cache= 1;
+  const_item_cache= true;
   for (uint32_t i= 0; i < arg_count; i++)
   {
     items[i]->update_used_tables();
@@ -130,7 +130,7 @@
 void Item_row::fix_after_pullout(Select_Lex *new_parent, Item **)
 {
   used_tables_cache= 0;
-  const_item_cache= 1;
+  const_item_cache= true;
   for (uint32_t i= 0; i < arg_count; i++)
   {
     items[i]->fix_after_pullout(new_parent, &items[i]);

=== modified file 'drizzled/item/subselect.cc'
--- drizzled/item/subselect.cc	2011-01-05 23:58:29 +0000
+++ drizzled/item/subselect.cc	2011-01-06 05:17:09 +0000
@@ -244,7 +244,7 @@
 
   if (engine->uncacheable())
   {
-    const_item_cache= 0;
+    const_item_cache= false;
     if (engine->uncacheable(UNCACHEABLE_RAND))
     {
       used_tables_cache|= RAND_TABLE_BIT;
@@ -394,7 +394,7 @@
   {
     // did all used tables become static?
     if (!(used_tables_cache & ~engine->upper_select_const_tables()))
-      const_item_cache= 1;
+      const_item_cache= true;
   }
 }
 

=== modified file 'drizzled/plugin/function.cc'
--- drizzled/plugin/function.cc	2010-12-03 18:46:31 +0000
+++ drizzled/plugin/function.cc	2011-01-06 05:17:09 +0000
@@ -25,6 +25,7 @@
 
 #include <drizzled/gettext.h>
 #include "drizzled/plugin/function.h"
+#include <drizzled/function_container.h>
 
 #include "drizzled/util/string.h"
 
@@ -40,6 +41,14 @@
 
 bool plugin::Function::addPlugin(const plugin::Function *udf)
 {
+  if (FunctionContainer::getMap().find(udf->getName()) != FunctionContainer::getMap().end())
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR,
+                  _("A function named %s already exists!\n"),
+                  udf->getName().c_str());
+    return true;
+  }
+
   if (udf_registry.find(udf->getName()) != udf_registry.end())
   {
     errmsg_printf(ERRMSG_LVL_ERROR,
@@ -47,6 +56,7 @@
                   udf->getName().c_str());
     return true;
   }
+
   std::pair<UdfMap::iterator, bool> ret=
     udf_registry.insert(make_pair(udf->getName(), udf));
   if (ret.second == false)
@@ -55,6 +65,7 @@
                   _("Could not add Function!\n"));
     return true;
   }
+
   return false;
 }
 

=== modified file 'drizzled/plugin/function.h'
--- drizzled/plugin/function.h	2010-12-18 04:43:40 +0000
+++ drizzled/plugin/function.h	2011-01-06 05:17:09 +0000
@@ -69,13 +69,10 @@
    */
   static void removePlugin(const plugin::Function *function_obj);
 
-  /**
-   * Accept a new connection (Protocol object) on one of the configured
-   * listener interfaces.
-   */
   static const plugin::Function *get(const char *name, size_t len=0);
 
   typedef boost::unordered_map<std::string, const plugin::Function *, util::insensitive_hash, util::insensitive_equal_to> UdfMap;
+  typedef boost::unordered_map<std::string, const plugin::Function *, util::insensitive_hash, util::insensitive_equal_to> Map;
 
   static const UdfMap &getMap();
 };

